aboutsummaryrefslogtreecommitdiffstats
path: root/src/vnet/l2/l2_learn.c
diff options
context:
space:
mode:
authorJohn Lo <loj@cisco.com>2017-08-03 00:35:36 -0400
committerNeale Ranns <nranns@cisco.com>2017-08-03 08:45:32 +0000
commit8d00fff8dff4e449767601645422e03df92a83af (patch)
treeba16f948bcd2bb65dec310876c5febd84fc85665 /src/vnet/l2/l2_learn.c
parenta825c8bf5c641051b0cf71c63c3ddd269d90069b (diff)
Add support for API client to receive L2 MAC events
Added APIs want_l2_macs_events and l2_macs_event to allow an API client to receive notification events from VPP for MAC learned or aged in L2FIB. Only one API client is allowed for L2 MAC events. The want_l2_macs_events API allow caller to specify MAC learn limit, event scan delay and max number of MACs that can be included in a event message. These parameters should be choosen properly as to not have too many MAC events sent by VPP and overwhelm the API share memory. They can all be left as 0's so VPP will setup reasonable defaults which are: 1000 learn limit, 100 msec scan delay and 100 MACs per event message. If want_l2_macs_events is never called, VPP learning and aging should behave as before except that MAC entries provisioned by API or CLI will not be aged, even if it is not set as static_mac. These non static MACs, however, can be overwritten by MAC learning on a MAC move as a leared MAC. Only learned MACs are subject to aging. Change-Id: Ia3757a80cf8adb2811a089d2eafbd6439461285c Signed-off-by: John Lo <loj@cisco.com>
Diffstat (limited to 'src/vnet/l2/l2_learn.c')
-rw-r--r--src/vnet/l2/l2_learn.c30
1 files changed, 20 insertions, 10 deletions
diff --git a/src/vnet/l2/l2_learn.c b/src/vnet/l2/l2_learn.c
index 65406292cc7..47c036b0bf1 100644
--- a/src/vnet/l2/l2_learn.c
+++ b/src/vnet/l2/l2_learn.c
@@ -123,11 +123,9 @@ l2learn_process (vlib_node_runtime_t * node,
/* Check mac table lookup result */
if (PREDICT_TRUE (result0->fields.sw_if_index == sw_if_index0))
{
/*
* The entry was in the table, and the sw_if_index matched, the normal case
*/
counter_base[L2LEARN_ERROR_HIT] += 1;
- int update = !result0->fields.static_mac &&
+ int update = !result0->fields.age_not && /* static_mac always age_not */
(result0->fields.timestamp != timestamp ||
result0->fields.sn.as_u16 != vnet_buffer (b0)->l2.l2fib_sn);
@@ -136,10 +134,10 @@ l2learn_process (vlib_node_runtime_t * node,
}
else if (result0->raw == ~0)
{
/* The entry was not in table, so add it */
counter_base[L2LEARN_ERROR_MISS] += 1;
- if (msm->global_learn_count == msm->global_learn_limit)
+ if (msm->global_learn_count >= msm->global_learn_limit)
{
/*
* Global limit reached. Do not learn the mac but forward the packet.
@@ -149,15 +147,22 @@ l2learn_process (vlib_node_runtime_t * node,
return;
}
+ /* Do not learn if mac is 0 */
+ l2fib_entry_key_t key = *key0;
+ key.fields.bd_index = 0;
+ if (key.raw == 0)
+ return;
+
/* It is ok to learn */
msm->global_learn_count++;
result0->raw = 0; /* clear all fields */
result0->fields.sw_if_index = sw_if_index0;
+ result0->fields.lrn_evt = (msm->client_pid != 0);
cached_key->raw = ~0; /* invalidate the cache */
}
else
{
/* The entry was in the table, but with the wrong sw_if_index mapping (mac move) */
if (result0->fields.filter)
{
ASSERT (result0->fields.sw_if_index == ~0);
@@ -167,8 +172,6 @@ l2learn_process (vlib_node_runtime_t * node,
return;
}
- counter_base[L2LEARN_ERROR_MAC_MOVE] += 1;
-
if (result0->fields.static_mac)
{
/*
@@ -185,6 +188,13 @@ l2learn_process (vlib_node_runtime_t * node,
* TODO: check global/bridge domain/interface learn limits
*/
result0->fields.sw_if_index = sw_if_index0;
+ if (result0->fields.age_not) /* The mac was provisioned */
+ {
+ msm->global_learn_count++;
+ result0->fields.age_not = 0;
+ }
+ result0->fields.lrn_evt = (msm->client_pid != 0);
+ counter_base[L2LEARN_ERROR_MAC_MOVE] += 1;
}
/* Update the entry */
@@ -479,7 +489,7 @@ VLIB_NODE_FUNCTION_MULTIARCH (l2learn_node, l2learn_node_fn)
* Set the default number of dynamically learned macs to the number
* of buckets.
*/
- mp->global_learn_limit = L2FIB_NUM_BUCKETS * 16;
+ mp->global_learn_limit = L2LEARN_DEFAULT_LIMIT;
return 0;
}
@@ -123,11 +123,9 @@ l2learn_process (vlib_node_runtime_t * node,
if (PREDICT_TRUE (result0->fields.sw_if_index == sw_if_index0))
{
- /*
- * The entry was in the table, and the sw_if_index matched, the normal case
- */
+ /* Entry in L2FIB with matching sw_if_index matched - normal fast path */
counter_base[L2LEARN_ERROR_HIT] += 1;
@@ -136,10 +134,10 @@ l2learn_process (vlib_node_runtime_t * node,
else if (result0->raw == ~0)
{
- /* The entry was not in table, so add it */
+ /* Entry not in L2FIB - add it */
counter_base[L2LEARN_ERROR_MISS] += 1;
@@ -149,15 +147,22 @@ l2learn_process (vlib_node_runtime_t * node,
else
{
- /* The entry was in the table, but with the wrong sw_if_index mapping (mac move) */
+ /* Entry in L2FIB with different sw_if_index - mac move or filter */
if (result0->fields.filter)
{